import { error, warn } from "../core/logger/core.js";
const STABLE = "stable";
const DEPRECATED = "deprecated";
const UNSUPPORTED = "unsupported";
const EXPERIMENTAL = "experimental";
const UNSUPPORTED_ASSETS_FEATURE = {
  supportKind: UNSUPPORTED,
  isSquooshCompatible: false,
  isSharpCompatible: false
};
const ALL_UNSUPPORTED = {
  serverOutput: UNSUPPORTED,
  staticOutput: UNSUPPORTED,
  hybridOutput: UNSUPPORTED,
  assets: UNSUPPORTED_ASSETS_FEATURE
};
function validateSupportedFeatures(adapterName, featureMap = ALL_UNSUPPORTED, config, logging) {
  const {
    assets = UNSUPPORTED_ASSETS_FEATURE,
    serverOutput = UNSUPPORTED,
    staticOutput = UNSUPPORTED,
    hybridOutput = UNSUPPORTED
  } = featureMap;
  const validationResult = {};
  validationResult.staticOutput = validateSupportKind(
    staticOutput,
    adapterName,
    logging,
    "staticOutput",
    () => config?.output === "static"
  );
  validationResult.hybridOutput = validateSupportKind(
    hybridOutput,
    adapterName,
    logging,
    "hybridOutput",
    () => config?.output === "hybrid"
  );
  validationResult.serverOutput = validateSupportKind(
    serverOutput,
    adapterName,
    logging,
    "serverOutput",
    () => config?.output === "server"
  );
  validationResult.assets = validateAssetsFeature(assets, adapterName, config, logging);
  return validationResult;
}
function validateSupportKind(supportKind, adapterName, logging, featureName, hasCorrectConfig) {
  if (supportKind === STABLE) {
    return true;
  } else if (supportKind === DEPRECATED) {
    featureIsDeprecated(adapterName, logging);
  } else if (supportKind === EXPERIMENTAL) {
    featureIsExperimental(adapterName, logging);
  }
  if (hasCorrectConfig() && supportKind === UNSUPPORTED) {
    featureIsUnsupported(adapterName, logging, featureName);
    return false;
  } else {
    return true;
  }
}
function featureIsUnsupported(adapterName, logging, featureName) {
  error(
    logging,
    `${adapterName}`,
    `The feature ${featureName} is not supported by the adapter ${adapterName}.`
  );
}
function featureIsExperimental(adapterName, logging) {
  warn(logging, `${adapterName}`, "The feature is experimental and subject to issues or changes.");
}
function featureIsDeprecated(adapterName, logging) {
  warn(
    logging,
    `${adapterName}`,
    "The feature is deprecated and will be moved in the next release."
  );
}
const SHARP_SERVICE = "astro/assets/services/sharp";
const SQUOOSH_SERVICE = "astro/assets/services/squoosh";
function validateAssetsFeature(assets, adapterName, config, logging) {
  const {
    supportKind = UNSUPPORTED,
    isSharpCompatible = false,
    isSquooshCompatible = false
  } = assets;
  if (config?.image?.service?.entrypoint === SHARP_SERVICE && !isSharpCompatible) {
    error(
      logging,
      "astro",
      `The currently selected adapter \`${adapterName}\` is not compatible with the image service "Sharp".`
    );
    return false;
  }
  if (config?.image?.service?.entrypoint === SQUOOSH_SERVICE && !isSquooshCompatible) {
    error(
      logging,
      "astro",
      `The currently selected adapter \`${adapterName}\` is not compatible with the image service "Squoosh".`
    );
    return false;
  }
  return validateSupportKind(supportKind, adapterName, logging, "assets", () => true);
}
export {
  validateSupportedFeatures
};
